package com.telelogic.rhapsody.wsdlTransformation;

import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.Frame;
import java.awt.GraphicsConfiguration;
import java.awt.Toolkit;

import com.telelogic.rhapsody.core.*;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.UIManager;
import javax.wsdl.Definition;
import javax.wsdl.WSDLException;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.xml.WSDLWriter;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import com.telelogic.rhapsody.rhpServicesTranslator.servicesModelElements.SRVPackage;
import com.telelogic.rhapsody.rhpWSDLTranslator.wsdlModelElements.*;
import com.telelogic.rhapsody.rhpXSDTranslator.domElements.DOMDocumentBuilder;
import com.telelogic.rhapsody.rhpXSDTranslator.utils.rhpUtils.RhapsodyUtils;
import com.telelogic.rhapsody.rhpXSDTranslator.xsdElements.XSDProfile;
import com.telelogic.rhapsody.rhpXSDTranslator.utils.guiUtils.ProgressBarIndicator;


public class WSDLGenerator extends RPUserPlugin {
	
	enum EPackageType {WSDL, Services, Undefined};
	
	static String generateWsdlSpecBaseMessage = "Select Output WSDL Specification File "; 
	
	static IRPApplication rhpApp = null;
	
	XSDProfile xsdProfile = null;


	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		//create an instance of my plugin
		WSDLGenerator myPlugin = new WSDLGenerator();
		//get Rhapsody application that is currently running
	      IRPApplication app                                                
	              =RhapsodyAppServer.getActiveRhapsodyApplication();
		//init the plugin
		myPlugin.RhpPluginInit(app);
		//imitate a call to the plugin
		myPlugin.OnMenuItemSelect("");
						
	}
	
	//called when the plug-in is loaded
	public void RhpPluginInit(final IRPApplication rpyApplication) {
		WSDLGenerator.rhpApp = rpyApplication;				
	}
	
	//called when the plug-in menu item under the "Tools" menu is selected
	public void RhpPluginInvokeItem() {
		final String INCORRECT_ELEM_SELECTED = "WSDL Generation is applied on a package stereotyped servicePackage or on a wsdlDefinitionDocument.\n Please select such an element in the browser and try again.";
		IRPModelElement selElem = WSDLGenerator.rhpApp.getSelectedElement();
		boolean selectionOK = false;
		if (selElem instanceof IRPPackage)
		{
			
			IRPPackage rhpPkg = (IRPPackage)selElem;
			EPackageType packageType = getPackageType(rhpPkg);
			if ((packageType == EPackageType.WSDL) || (packageType == EPackageType.Services))					
			{
				selectionOK = true;
				exportPackageFromRhapsody(rhpPkg);			
			}
		}
		if (selectionOK == false)
		{
			JFrame frame = createDummyFrame();
			JOptionPane.showMessageDialog(frame, INCORRECT_ELEM_SELECTED, "WSDL Generation", JOptionPane.ERROR_MESSAGE);
			frame.dispose();
		}
	}

	
	private void exportPackageFromRhapsody(IRPPackage rhpPkg)
	{
		ProgressBarIndicator progressBarIndicator = new ProgressBarIndicator("WSDL Generation");		
		String specFilename = getSpecificationsFile(rhpPkg);			
		EPackageType packageType = getPackageType(rhpPkg);
		boolean abortGeneration = false;
		if (specFilename.equals("nullnull") == true)
		{
			abortGeneration = true;
	    }
		try
		{		
				if (abortGeneration == false)
				{
					progressBarIndicator.setVisible(true);
					//Initialize the profile types.					
					xsdProfile = XSDProfile.instance();
					switch (packageType)
					{
						case WSDL:
							exportWSDLFromRhapsody(rhpPkg, specFilename);
							break;
						case Services:
							exportServiceFromRhapsody(rhpPkg, specFilename);
							break;					
					}
					progressBarIndicator.setVisible(false);
				}				    					    	
		}
		catch (RhapsodyRuntimeException e)
		{			 
			String title = "Error on call Rhapsody API:" + rhpApp.getErrorMessage();
			reportExceptionToLogWindow(e, title);
		}	
		catch (IllegalArgumentException e)
		{
			String title = "IllegalArgumentException:";
			reportExceptionToLogWindow(e, title);
		}
		catch (Exception e)
		{
			String title = "General exception:";
			reportExceptionToLogWindow(e, title);
		}		
		progressBarIndicator.dispose();		
	}
	
	private void reportExceptionToLogWindow(Exception e, String title)
	{
		String errorMessage =  title  + " - exception message: " +  e.getMessage() + "\n";
		RhapsodyUtils.writeToOutputWindow("Log", errorMessage);
	}
	
	private void exportWSDLFromRhapsody(IRPPackage rhpPkg, String specFilename)
	{
		DOMDocumentBuilder.clearDocument();
		WSDLDefinition wsdlDefinition = new WSDLDefinition(rhpPkg);
		wsdlDefinition.init();
		wsdlDefinition.resolve();
		wsdlDefinition.exportFromRhapsody(null);
		wsdlDefinition.resolveInTargetModel();
		Definition definition = WSDLDefinition.getDefinition();
		writeWSDL(definition, specFilename);
		String doneMessage = "Generation of WSDL specification " + specFilename + " is done.\n";
		rhpApp.writeToOutputWindow("Log", doneMessage);		
	}
	
	private void exportServiceFromRhapsody(IRPPackage rhpPkg, String specFilename)
	{
		IRPProject rhpPrj = rhpApp.activeProject();
		SRVPackage.setRhpServicesPackage(rhpPkg);
		SRVPackage srvPkg = SRVPackage.instance();
		srvPkg.resolve();		
		IRPPackage rhpWSDLPkg = (IRPPackage)srvPkg.exportFromRhapsody(rhpPrj);
		srvPkg.resolveInTargetModel();
		SRVPackage.clearRhpToTransElementMap();
		//After clearing the rhpToTransElementMap, add again the profile types so they can be used by the WSDL transformation.
		xsdProfile.updateRhpToTransElementMap();
		exportWSDLFromRhapsody(rhpWSDLPkg, specFilename);
		SRVPackage.clearInstance();
		//Delete the WSDL package - it was created only for the WSDL specification generation.
		rhpWSDLPkg.deletePackage();
	}
	
	
	private EPackageType getPackageType(IRPPackage rhpPkg)
	{
		EPackageType res = EPackageType.Undefined;
		if (RhapsodyUtils.hasStereotype(rhpPkg, "wsdlDefinitionDocument") == true)
		{
			res = EPackageType.WSDL;			
		}
		else if (RhapsodyUtils.hasStereotype(rhpPkg, "servicePackage") == true)
		{
			res = EPackageType.Services;
		}
		return res;
	}
	
	private String getSpecificationsFile(IRPPackage rhpPkg)
	{
		String fileName = null; 
		String title = generateWsdlSpecBaseMessage + "For " + rhpPkg.getName();
		Frame ownerFrame = null;
		try
		{
			ownerFrame = new Frame((GraphicsConfiguration)null);
		}
		catch (IllegalArgumentException e)
		{
			reportExceptionToLogWindow(e, "Unable to create default frame for file dialog");			
		}
		if (ownerFrame != null)
		{
			FileDialog fileDialog = new FileDialog(ownerFrame, title, FileDialog.SAVE);
			fileDialog.setVisible(true);
			fileDialog.toFront();
			fileName = fileDialog.getDirectory() + fileDialog.getFile();
			ownerFrame.dispose();
			ownerFrame = null;
		}
		return fileName;		
	}
	
	//called when the project is closed
	public boolean RhpPluginCleanup(){
		return true;
	}
	
	//called when Rhapsody exits
	public void RhpPluginFinalCleanup(){
		WSDLGenerator.rhpApp = null;		
	}
	
	private void writeWSDL(Definition wsdlDefinition, String outputFilename) 
	{
		try {
			WSDLWriter wsdlWriter = WSDLFactory.newInstance().newWSDLWriter();
			FileOutputStream fos = new FileOutputStream(outputFilename); 
			wsdlWriter.writeWSDL(wsdlDefinition, fos);
			fos.close();
		} catch (WSDLException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void OnMenuItemSelect(String menuItem) {
		// TODO Auto-generated method stub
		final IRPModelElement selElem = WSDLGenerator.rhpApp.getSelectedElement();
		IRPPackage rhpPkg = (IRPPackage)selElem;
		exportPackageFromRhapsody(rhpPkg);			
	}

	@Override
	public void OnTrigger(String trigger) {
		// TODO Auto-generated method stub
		
	}

	private static JFrame createDummyFrame()
	{
		try
		{
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());	
		}
		catch (Exception e)
		{
			
		}
		JFrame frame = new JFrame("Import WSDL");
		frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		frame.setSize(new Dimension(10,10));
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		
		int xCenter = (screen.width / 2) - (frame.getWidth() / 2); // Center
		int yCenter = (screen.height / 2) - (frame.getHeight() / 2); // Center
		frame.setLocation(xCenter, yCenter);
		frame.toFront();
		frame.setAlwaysOnTop(true);
		frame.setVisible(true);
		frame.setVisible(false);
		return frame;
	}
	

}





	
	
	

