package com.telelogic.rhapsody.wsdlTransformation;

import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.Frame;
import java.awt.GraphicsConfiguration;
import java.awt.Toolkit;
import java.util.Iterator;
import java.util.List;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.UIManager;

import com.telelogic.rhapsody.core.*;
import javax.wsdl.Definition;
import javax.wsdl.WSDLException;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.xml.WSDLReader;
import com.telelogic.rhapsody.rhpWSDLTranslator.wsdlModelElements.*;
import com.telelogic.rhapsody.rhpXSDTranslator.utils.rhpUtils.RhapsodyUtils;
import com.telelogic.rhapsody.rhpXSDTranslator.utils.guiUtils.ProgressBarIndicator;

public class WSDLImporter extends RPUserPlugin {

	static String importWsdlSpecMessage = "Select WSDL Specification File"; 
	
	static IRPApplication rhpApp = null;
	//the plugin extended classes factory
	
		
	public static void main(String[] args) {
		rhpApp = RhapsodyAppServer.getActiveRhapsodyApplication();
		importToRhapsody();
	}
	
	
	//called when the plug-in is loaded
	public void RhpPluginInit(final IRPApplication rpyApplication) {
		WSDLImporter.rhpApp = rpyApplication;		
	}
	
	//called when the plug-in menu item under the "Tools" menu is selected
	public void RhpPluginInvokeItem() {
		//JOptionPane.showMessageDialog(null, "Tools menu called");
		importToRhapsody();		
	}
	
	private static void importToRhapsody()
	{
		IRPModelElement selElem = WSDLImporter.rhpApp.getSelectedElement();
		boolean abortExport = false;
		ProgressBarIndicator progressBarIndicator = new ProgressBarIndicator("WSDL import");		
		try
		{	
			if ((selElem == null) || !(selElem instanceof IRPPackage))
			{
				selElem = rhpApp.activeProject();
			}
			IRPPackage rhpSelPkg = (IRPPackage)selElem; 
			String filename = getSpecificationsFile();			
			Definition definition = null;
			if (filename.equals("nullnull") == false)
			{
				progressBarIndicator.setVisible(true);
				definition = readWSDL(filename);
				if (definition != null)
				{
					WSDLDefinition wsdlDefinition = new WSDLDefinition(definition);
					wsdlDefinition.init();
					String wsdlPackageName = wsdlDefinition.getName();
					IRPPackage nestedPackage = getNestedPackage(rhpSelPkg, wsdlPackageName); 
					if (nestedPackage != null)
					{
						progressBarIndicator.setVisible(false);
						String message = "A package with the name " + wsdlPackageName + " already exist in the selected location. Do you wish to replace the package ?";
						JFrame frame = createDummyFrame();
						int shouldReplace = JOptionPane.showConfirmDialog(frame, 
								message, message, JOptionPane.YES_NO_OPTION);
						frame.dispose();
						if (shouldReplace == JOptionPane.YES_OPTION)
						{
							//try to delete the package
							try
							{
								nestedPackage.deletePackage();
							}
							catch (Exception e)
							{
								String errorMessage = e.getMessage();
								JFrame frame1 = createDummyFrame();
								JOptionPane.showMessageDialog(frame1, "Failed to delete the package due to the following exception" + errorMessage);
								frame1.dispose();
								abortExport = true;
							}
						}
						else
						{
							abortExport = true;
						}
						
					}
					if (abortExport == false)
					{
						progressBarIndicator.setVisible(true);
						wsdlDefinition.resolve();
						wsdlDefinition.importToRhapsody(selElem);
						wsdlDefinition.resolveInRhapsody(selElem);
						progressBarIndicator.setVisible(false);
						String doneMessage = "Import of WSDL specification " + filename + " is done.\n";
						rhpApp.writeToOutputWindow("Log", doneMessage);
					}
				}
				else
				{
					rhpApp.writeToOutputWindow("Log", "Failed to read file " + filename + "\n");					
				}				
			}
		}
		catch (RhapsodyRuntimeException e)
		{
			String errorMessage = rhpApp.getErrorMessage();
			rhpApp.writeToOutputWindow("Log", "error on JavaAPI - " + errorMessage + "\n");
		}
		catch (Exception e)
		{
			reportExceptionToLogWindow(e, "General error");						
		}
		progressBarIndicator.dispose();		
	}
	
	private static String getSpecificationsFile()
	{
		String fileName = null; 		
		Frame ownerFrame = null;
		try
		{
			ownerFrame = new Frame((GraphicsConfiguration)null);
		}
		catch (IllegalArgumentException e)
		{
			reportExceptionToLogWindow(e, "Unable to create default frame for file dialog");			
		}
		if (ownerFrame != null)
		{
			FileDialog fileDialog = new FileDialog(ownerFrame, importWsdlSpecMessage, FileDialog.LOAD);
			fileDialog.setVisible(true);
			fileDialog.toFront();
			fileName = fileDialog.getDirectory() + fileDialog.getFile();
			ownerFrame.dispose();
			ownerFrame = null;
		}
		return fileName;				
	}
	
	@SuppressWarnings("unchecked")
	private static IRPPackage getNestedPackage(IRPPackage rhpPkg, String nestedPkgName)
	{
		IRPPackage res = null;
		IRPCollection pkgColl = rhpPkg.getPackages();
		List pkgs = pkgColl.toList();
		Iterator pkgIter = pkgs.iterator();
		while (pkgIter.hasNext() && (res == null))
		{
			IRPPackage rhpTracedPkg = (IRPPackage)pkgIter.next();
			String tracedPkgName = rhpTracedPkg.getName();
			if (tracedPkgName.equals(nestedPkgName) == true)
			{
				res = rhpTracedPkg;
			}
		}
		return res;		
	}
	
	//called when the project is closed
	public boolean  RhpPluginCleanup(){
		return true;		
	}
	
	//called when Rhapsody exits
	public void RhpPluginFinalCleanup(){
		WSDLImporter.rhpApp = null;
		
	}
	
	private static Definition readWSDL(String filename) 
	{
		Definition res = null;
		try {
			WSDLReader wsdlReader = WSDLFactory.newInstance().newWSDLReader();
			res = wsdlReader.readWSDL(null, filename);
			
		} catch (WSDLException e) {	
			JOptionPane.showMessageDialog(null, e);
		}
		return res;
	}


	@Override
	public void OnMenuItemSelect(String menuItem) {
		// TODO Auto-generated method stub
		importToRhapsody();		
	}


	@Override
	public void OnTrigger(String trigger) {
		// TODO Auto-generated method stub
		
	}
	
	private static JFrame createDummyFrame()
	{
		try
		{
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());	
		}
		catch (Exception e)
		{
			
		}
		JFrame frame = new JFrame("Import WSDL");
		frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		frame.setSize(new Dimension(10,10));
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		
		int xCenter = (screen.width / 2) - (frame.getWidth() / 2); // Center
		int yCenter = (screen.height / 2) - (frame.getHeight() / 2); // Center
		frame.setLocation(xCenter, yCenter);
		frame.toFront();
		frame.setAlwaysOnTop(true);
		frame.setVisible(true);
		frame.setVisible(false);
		return frame;
	}


	private static void reportExceptionToLogWindow(Exception e, String title)
	{
		String exceptionErrorMessage = e.getMessage();
		String errorMessage;
		if (exceptionErrorMessage == null)
		{
			errorMessage =  title  + " - no available error message.\n";
		}
		else
		{
			errorMessage =  title  + " - exception message: " +  exceptionErrorMessage + "\n";	
		}		
		RhapsodyUtils.writeToOutputWindow("Log", errorMessage);
		RhapsodyUtils.writeToOutputWindow("Log", "Failed to import file\n");
	}
	

}
