package com.sodius.mdw.rhapsody.api.utils;

import com.sodius.mdw.metamodel.rhapsody.ModelElement;

/**
 * This interface is used to customize filenames associated to <b>IBM Rational Rhapsody</b> objects.<p/>
 * For each object, we can propose several files that <b>IBM Rational Rhapsody</b> could be able to bind and edit on user demand.<p/>
 * To activate this process in the current <b>Rhapsody project</b>, you have to change the Rhapsody property of the current active configuration in the <b>Features</b> window:<br/>
 * <code><i>lang_CG</i>:Configuration:ExternalGeneratorFileMappingRules</code> = <b>DefinedByGenerator</b>
 */
public interface RhapsodyFileMapper {
	/**
	 * Add a filename to an element association list<p/>
	 * Only 2 files are accepted by code generation, beyond the list is emptied.
	 * @param element A <b>ModelElement-based</b> object except <b>Configuration</b>, for example: <b>Package</b>, <b>Class</b>, <b>Attribute</b>, <b>Operation</b>, etc... 
	 * @param filename	full path filename
	 * @return <b>filename</b> parameter value
	 */
	public String addFileName(ModelElement element, String filename);

	/**
	 * Define the single main file associated to the active configuration object
	 * @param element the active <b>Configuration</b> object only 
	 * @param filename	full path filename
	 * @return <b>filename</b> parameter value
	 */
	public String addMainFileName(ModelElement element, String filename);

	/**
	 * Define the single makefile associated to the active configuration object
	 * @param element the active <b>Configuration</b> object only 
	 * @param filename	full path filename
	 * @return <b>filename</b> parameter value
	 */
	public String addMakeFileName(ModelElement element, String filename);


	/**
	 * Define the single target file (built application or library) associated to the active configuration object
	 * @param element the active <b>Configuration</b> object only 
	 * @param filename	full path filename
	 * @return <b>filename</b> parameter value
	 */
	public String addTargetFileName(ModelElement element, String filename);
}
