
//Licensed Materials - Property of IBM 
// Copyright IBM Corporation 2006, 2008. All Rights Reserved.

package com.telelogic.rhapsody.core;



public interface IRPModelElement {
	/**
	 * method addAssociation
	 * @throws RhapsodyRuntimeException
	 */
	public IRPAssociationClass addAssociation(IRPRelation end1, IRPRelation end2, String name);
	/**
	 * method addDependency
	 * @throws RhapsodyRuntimeException
	 */
	public IRPDependency addDependency(String dependsOnName, String dependsOnType);
	/**
	 * method addDependencyTo
	 * @throws RhapsodyRuntimeException
	 */
	public IRPDependency addDependencyTo(IRPModelElement element);
	/**
	 * method addNewAggr
	 * @throws RhapsodyRuntimeException
	 */
	public IRPModelElement addNewAggr(String metaType, String name);
	/**
	 * method addProperty
	 * @throws RhapsodyRuntimeException
	 */
	public void addProperty(String propertyKey, String propertyType, String propertyValue);
	/**
	 * method addSpecificStereotype
	 * @throws RhapsodyRuntimeException
	 */
	public void addSpecificStereotype(IRPStereotype stereotype);
	/**
	 * method addStereotype
	 * @throws RhapsodyRuntimeException
	 */
	public IRPStereotype addStereotype(String name, String metaType);
	/**
	 * becomeTemplateInstantiationOf
	 * @throws RhapsodyRuntimeException
	 */
	public void becomeTemplateInstantiationOf(IRPModelElement newVal);
	/**
	 * method clone
	 * @throws RhapsodyRuntimeException
	 */
	public IRPModelElement clone(String name, IRPModelElement newOwner);
	/**
	 * method deleteDependency
	 * @throws RhapsodyRuntimeException
	 */
	public void deleteDependency(IRPDependency dependency);
	/**
	 * method deleteFromProject
	 * @throws RhapsodyRuntimeException
	 */
	public void deleteFromProject();
	/**
	 * method errorMessage
	 * @throws RhapsodyRuntimeException
	 */
	public String errorMessage();
	/**
	 * method findElementsByFullName
	 * @throws RhapsodyRuntimeException
	 */
	public IRPModelElement findElementsByFullName(String name, String metaClass);
	/**
	 * method findNestedElement
	 * @throws RhapsodyRuntimeException
	 */
	public IRPModelElement findNestedElement(String name, String metaClass);
	/**
	 * method findNestedElementRecursive
	 * @throws RhapsodyRuntimeException
	 */
	public IRPModelElement findNestedElementRecursive(String name, String metaClass);
	/**
	 * get property allTags
	 * @throws RhapsodyRuntimeException
	 */
	public IRPCollection getAllTags();
	/**
	 * get property annotations
	 * @throws RhapsodyRuntimeException
	 */
	public IRPCollection getAnnotations();
	/**
	 * get property associationClasses
	 * @throws RhapsodyRuntimeException
	 */
	public IRPCollection getAssociationClasses();
	/**
	 * get property constraints
	 * @throws RhapsodyRuntimeException
	 */
	public IRPCollection getConstraints();
	/**
	 * get property constraintsByHim
	 * @throws RhapsodyRuntimeException
	 */
	public IRPCollection getConstraintsByHim();
	/**
	 * get property controlledFiles
	 * @throws RhapsodyRuntimeException
	 */
	public IRPCollection getControlledFiles();
	/**
	 * get property dependencies
	 * @throws RhapsodyRuntimeException
	 */
	public IRPCollection getDependencies();
	/**
	 * get property description
	 * @throws RhapsodyRuntimeException
	 */
	public String getDescription();
	/**
	 * get property description in HTML format
	 * @throws RhapsodyRuntimeException
	 */
	public String getDescriptionHTML();
	/**
	 * get property description in RTF format
	 * @throws RhapsodyRuntimeException
	 */
	public String getDescriptionRTF();
	/**
	 * get property displayName
	 * @throws RhapsodyRuntimeException
	 */
	public String getDisplayName();
	/**
	 * property displayName in RTF format
	 * @throws RhapsodyRuntimeException
	 */
	public String getDisplayNameRTF();
	/**
	 * method getErrorMessage
	 * @throws RhapsodyRuntimeException
	 */
	public String getErrorMessage();
	/**
	 * method getFullPathName
	 * @throws RhapsodyRuntimeException
	 */
	public String getFullPathName();
	/**
	 * method getFullPathNameIn
	 * @throws RhapsodyRuntimeException
	 */
	public String getFullPathNameIn();
	/**
	 * get property GUID
	 * @throws RhapsodyRuntimeException
	 */
	public String getGUID();
	/**
	 * get property hyperLinks
	 * @throws RhapsodyRuntimeException
	 */
	public IRPCollection getHyperLinks();
	/**
	 * Get the full path of the element's icon
	 * @throws RhapsodyRuntimeException
	 */
	public String getIconFileName();
	/**
	 * get property interfaceName
	 * @throws RhapsodyRuntimeException
	 */
	public String getInterfaceName();
	/**
	 * get property isExternal
	 * @throws RhapsodyRuntimeException
	 */
	public int getIsExternal();
	/**
	 * get property isOfMetaClass
	 * @throws RhapsodyRuntimeException
	 */
	public int getIsOfMetaClass(String metaClass);
	/**
	 * get property isShowDisplayName
	 * @throws RhapsodyRuntimeException
	 */
	public int getIsShowDisplayName();
	/**
	 * get property isUnresolved
	 * @throws RhapsodyRuntimeException
	 */
	public int getIsUnresolved();
	/**
	 * get property localTags
	 * @throws RhapsodyRuntimeException
	 */
	public IRPCollection getLocalTags();
	/**
	 * get property main diagram
	 * @throws RhapsodyRuntimeException
	 */
	public IRPDiagram getMainDiagram();
	/**
	 * get property metaClass
	 * @throws RhapsodyRuntimeException
	 */
	public String getMetaClass();
	/**
	 * get property name
	 * @throws RhapsodyRuntimeException
	 */
	public String getName();
	/**
	 * method getNestedElements
	 * @throws RhapsodyRuntimeException
	 */
	public IRPCollection getNestedElements();
	/**
	 * method getNestedElementsRecursive
	 * @throws RhapsodyRuntimeException
	 */
	public IRPCollection getNestedElementsRecursive();
	/**
	 * get property of template
	 * @throws RhapsodyRuntimeException
	 */
	public IRPModelElement getOfTemplate();
	/**
	 * Get the full path of the element's overlay icon
	 * @throws RhapsodyRuntimeException
	 */
	public String getOverlayIconFileName();
	/**
	 * method getOverriddenProperties
	 * @throws RhapsodyRuntimeException
	 */
	public IRPCollection getOverriddenProperties(int recursive);
	/**
	 * method getOverriddenPropertiesByPattern
	 * @throws RhapsodyRuntimeException
	 */
	public IRPCollection getOverriddenPropertiesByPattern(String pattern, int localyOverridenOnly, int withDefaultValues);
	/**
	 * get property owner
	 * @throws RhapsodyRuntimeException
	 */
	public IRPModelElement getOwner();
	/**
	 * get property project
	 * @throws RhapsodyRuntimeException
	 */
	public IRPProject getProject();
	/**
	 * method getPropertyValue
	 * @throws RhapsodyRuntimeException
	 */
	public String getPropertyValue(String propertyKey);
	/**
	 * method getPropertyValueExplicit
	 * @throws RhapsodyRuntimeException
	 */
	public String getPropertyValueExplicit(String propertyKey);
	/**
	 * get property references
	 * @throws RhapsodyRuntimeException
	 */
	public IRPCollection getReferences();
	/**
	 * get property requirementTraceabilityHandle
	 * @throws RhapsodyRuntimeException
	 */
	public int getRequirementTraceabilityHandle();
	/**
	 * Get the the save unit the element belongs to
	 * @throws RhapsodyRuntimeException
	 */
	public IRPUnit getSaveUnit();
	/**
	 * get property stereotype
	 * @throws RhapsodyRuntimeException
	 */
	public IRPStereotype getStereotype();
	/**
	 * get property stereotypes
	 * @throws RhapsodyRuntimeException
	 */
	public IRPCollection getStereotypes();
	/**
	 * method getTag
	 * @throws RhapsodyRuntimeException
	 */
	public IRPTag getTag(String name);
	/**
	 * get property templateParameters
	 * @throws RhapsodyRuntimeException
	 */
	public IRPCollection getTemplateParameters();
	/**
	 * get property template instantiation
	 * @throws RhapsodyRuntimeException
	 */
	public IRPTemplateInstantiation getTi();
	/**
	 * get property userDefinedMetaClass
	 * @throws RhapsodyRuntimeException
	 */
	public String getUserDefinedMetaClass();
	/**
	 * Check if the element has children
	 * @throws RhapsodyRuntimeException
	 */
	public int hasNestedElements();
	/**
	 * highlight element
	 * @throws RhapsodyRuntimeException
	 */
	public void highLightElement();
	/**
	 * method isATemplate
	 * @throws RhapsodyRuntimeException
	 */
	public int isATemplate();
	/**
	 * determine whether description is in RTF format
	 * @throws RhapsodyRuntimeException
	 */
	public int isDescriptionRTF();
	/**
	 * determine whether displayName is in RTF format
	 * @throws RhapsodyRuntimeException
	 */
	public int isDisplayNameRTF();
	/**
	 * Check if the element is modified
	 * @throws RhapsodyRuntimeException
	 */
	public int isModified();
	/**
	 * Locate the model element in the Browser
	 * @throws RhapsodyRuntimeException
	 */
	public int locateInBrowser();
	/**
	 * property openFeaturesDialog
	 * @throws RhapsodyRuntimeException
	 */
	public void openFeaturesDialog(int newDialog);
	/**
	 * method removeProperty
	 * @throws RhapsodyRuntimeException
	 */
	public void removeProperty(String propertyKey);
	/**
	 * method removeStereotype
	 * @throws RhapsodyRuntimeException
	 */
	public void removeStereotype(IRPStereotype stereotype);
	/**
	 * set property description
	 * @throws RhapsodyRuntimeException
	 */
	public void setDescription(String description);
	/**
	 * set property description in HTML format
	 * @throws RhapsodyRuntimeException
	 */
	public void setDescriptionHTML(String descriptionHTML);
	/**
	 * set property description in RTF format
	 * @throws RhapsodyRuntimeException
	 */
	public void setDescriptionRTF(String descriptionRTF);
	/**
	 * set property displayName
	 * @throws RhapsodyRuntimeException
	 */
	public void setDisplayName(String displayName);
	/**
	 * property displayName in RTF format
	 * @throws RhapsodyRuntimeException
	 */
	public void setDisplayNameRTF(String newVal);
	/**
	 * set property GUID
	 * @throws RhapsodyRuntimeException
	 */
	public void setGUID(String gUID);
	/**
	 * set property isShowDisplayName
	 * @throws RhapsodyRuntimeException
	 */
	public void setIsShowDisplayName(int isShowDisplayName);
	/**
	 * set property main diagram
	 * @throws RhapsodyRuntimeException
	 */
	public void setMainDiagram(IRPDiagram mainDiagram);
	/**
	 * set property name
	 * @throws RhapsodyRuntimeException
	 */
	public void setName(String name);
	/**
	 * set property of template
	 * @throws RhapsodyRuntimeException
	 */
	public void setOfTemplate(IRPModelElement ofTemplate);
	/**
	 * set property owner
	 * @throws RhapsodyRuntimeException
	 */
	public void setOwner(IRPModelElement owner);
	/**
	 * method setPropertyValue
	 * @throws RhapsodyRuntimeException
	 */
	public void setPropertyValue(String propertyKey, String propertyValue);
	/**
	 * set property requirementTraceabilityHandle
	 * @throws RhapsodyRuntimeException
	 */
	public void setRequirementTraceabilityHandle(int requirementTraceabilityHandle);
	/**
	 * set property stereotype
	 * @throws RhapsodyRuntimeException
	 */
	public void setStereotype(IRPStereotype stereotype);
	/**
	 * method setTagValue
	 * @throws RhapsodyRuntimeException
	 */
	public IRPTag setTagValue(IRPTag tag, String val);
	/**
	 * set property template instantiation
	 * @throws RhapsodyRuntimeException
	 */
	public void setTi(IRPTemplateInstantiation ti);
	/**
	 * synchronizeTemplateInstantiation
	 * @throws RhapsodyRuntimeException
	 */
	public void synchronizeTemplateInstantiation();
}

